unit skyperfdataTests;

interface

uses
  Classes,
  Contnrs,
  skyperfdata,
  TestFrameWork;

type
  TChargeFieldTests = class(TTestCase)
  private

  protected

//    procedure SetUp; override;
//    procedure TearDown; override;

  published

    // Test methods
    procedure TestCreate;

  end;

{
type
  TPackTests = class(TTestCase)
  private

  protected

//    procedure SetUp; override;
//    procedure TearDown; override;

  published

    // Test methods
    procedure TestAddChannel;
    procedure TestAddPack;
    procedure TestCanAddToResult;
    procedure TestUpdateChannels;
    procedure TestContains;
    procedure TestContainsChannelName;
    procedure TestContainsPack;
    procedure TestContainAllChannels;
    procedure TestCreate;
    procedure TestDestroy;
    procedure TestNextChannel;
    procedure TestGetCharge;
    procedure TestGetName;
    procedure TestNeeds;
    procedure TestLimited;
    procedure TestRootLimitedPack;

  end;
}
{
type
  TRequestTests = class(TTestCase)
  private

  protected

//    procedure SetUp; override;
//    procedure TearDown; override;

  published

    // Test methods
    procedure TestCreate;
    procedure TestClearSelections;
    procedure TestSetRequest;
    procedure TestGetRequest;
    procedure TestGetChannelCandidateCount;
    procedure TestGetChannelCount;
    procedure TestGetChannels;
    procedure TestGetChannelVariationCount;
    procedure TestSetServiceEnabled;
    procedure TestGetServiceEnabled;
    procedure TestDisableAllServices;
    procedure TestEnableAllServices;

  end;
}
{
type
  TResultTests = class(TTestCase)
  private

  protected

//    procedure SetUp; override;
//    procedure TearDown; override;

  published

    // Test methods
    procedure TestCreate;
    procedure TestSetRestChannels;
    procedure TestAddPack;
    procedure TestAddPackWithoutLimited;
    procedure TestAssign;
    procedure Testvalid;
    procedure TestHasLimit;
    procedure TestNeeds;
    procedure TestGetRestChannelsTotalCharge;
    procedure TestCanAddPack;
    procedure TestTestAddPack;
    procedure TestGetAssignedPack;

  end;
}
{
type
  TSkyPerfCalcTests = class(TTestCase)
  private

  protected

//    procedure SetUp; override;
//    procedure TearDown; override;

  published

    // Test methods
    procedure TestCreate;
    procedure TestDestroy;
    procedure TestCreateRequest;
    procedure TestEnumerate;
    procedure TestFindChannel;
    procedure TestFindChannelByName;
    procedure TestFindService;
    procedure TestFindPack;

  end;
}

type
  TSkyPerfCalcTests = class(TTestCase)
  private
    function CreateDataWithLimited: TStringList;
    function DoRequest(const calc: TSkyPerfCalc;
      const chs: array of integer): TObjectList;
    function CreateSimpleData: TStringList;
//    function GetResultText(const r: TResult): string;
    function CreateDataWithCombinationDiscount: TStringList;

  protected

//    procedure SetUp; override;
//    procedure TearDown; override;

  published

    // Test methods
//    procedure TestCreate;
//    procedure TestDestroy;
//    procedure TestCreateRequest;
//    procedure TestEnumerate;
//    procedure TestFindChannel;
//    procedure TestFindChannelByName;
//    procedure TestFindService;
//    procedure TestFindPack;

    procedure TestLoad;
    procedure TestEnumerateSimple1;
    procedure TestEnumerateSimple2;
    procedure TestEnumerateLimited123;
    procedure TestEnumerateLimited1234;
    procedure TestEnumerateLimited12345;
    procedure TestEnumerateCombinationDiscount;
  end;

  TCallback = class(TInterfacedObject, IMessageCallback)
  private
    FBuffer: string;
  public
    constructor Create;
    procedure Clear;
    function Get: string;
    procedure AddLine( s: string );
  end;

type
  TIntSetTests = class(TTestCase)
  private
    FObject: TIntSet;
  protected

    procedure SetUp; override;
    procedure TearDown; override;

  published

    // Test methods
    procedure TestCreate;
    procedure TestFirst_Last;
    procedure TestAdd_Count;
    procedure TestRemove;
    procedure TestClear;
    procedure TestAssign_Next;
    procedure TestAddSet_RemoveSet;
    procedure TestHas;

  end;

implementation
uses
  SysUtils;

function PriceCompare(Item1, Item2: Pointer): Integer;
begin
  result := TResult(Item1).totalcharge - TResult(Item2).totalcharge;
end;

function GetResultText( const r: TResult ): string;
var
  i: integer;
  sl: TStringList;
begin
  sl := TStringList.Create;
  result := '';

  for i := 0 to r.PackCount-1 do
  begin
    sl.Add(r.Packs[i].Name);
  end;
  for i := 0 to r.RestChannelCount-1 do
  begin
    sl.Add(IntToStr(r.RestChannels[i].Num));
  end;
  sl.Sort;
  result := sl.CommaText;
  sl.Free;
end;

function ResultTextCompare(Item1, Item2: Pointer): Integer;
begin
  result := CompareText(GetResultText(TResult(Item1)), GetResultText(TResult(Item2)));
end;

function FindResultText( var r: TResult; var results: TObjectList; s: string ): boolean;
var
  i: integer;
  j: integer;
begin
  for i := 0 to results.Count-1 do
  begin
    if GetResultText(results[i] as TResult) = s then
    begin
      r := results[i] as TResult;
      for j := i downto 1 do
        results.Exchange(j, j - 1);
      result := true;
      exit;
    end;
  end;
  r := nil;
  result := false;
end;

{ TSkyPerfCalcTests }

{ TODO : limited packŏeXg }

function TSkyPerfCalcTests.DoRequest(const calc: TSkyPerfCalc; const chs: array of integer): TObjectList;
  function CreateRequest(const calc: TSkyPerfCalc; const chs: array of integer): TRequest;
  var
    i: integer;
    ch: TChannel;
  begin
    result := TRequest.Create(calc);
    result.ClearSelections;
    result.EnableAllServices;
    for i := 0 to High(chs) do
    begin
      ch := calc.FindChannel(chs[i]);
      CheckTrue( ch <> nil, IntToStr(chs[i]) + 'ch' );
      result.SetRequest(ch, AnyService);
    end;
    CheckEquals( High(chs)+1, result.GetChannelCount, 'result.GetChannelCount' );
  end;

var
  request: TRequest;
  variation: integer;
begin
  result := TObjectList.Create;
  try
    result.OwnsObjects := true;

    request := CreateRequest(calc, chs);
    try
      for variation := 0 to request.GetChannelVariationCount-1 do
        calc.Enumerate(result, request, variation);
    finally
      request.Free;
    end;
    result.Sort(PriceCompare);
  except
    result := nil;
  end;
end;

function TSkyPerfCalcTests.CreateSimpleData: TStringList;
var
  data: TStringList;
begin
  data := TStringList.Create;
  data.Add(';comment');
  data.Add('S,1,100,0,Service1,123,SVC1');

//C,`lԍ,,(0ȂPƉs)
//C,`lԍ+bias, O, , sT[rX̑Ή`l(Ȃ0)
  data.Add('C,1,Ch1,0'); // Ch1(SVC1)͒PƉs
  data.Add('C,2,Ch2,100'); // Ch2(SVC1)100~

//P,pbN,,\`l/pbN,...
//P,pbN,,\pbN,*`l
  data.Add('P,Pack1,200,1,2');

  result := data;
end;

procedure TSkyPerfCalcTests.TestEnumerateSimple1;
var
  callback: IMessageCallback;
  data: TStringList;
  calc: TSkyPerfCalc;
  results: TObjectList;
begin
  callback := TCallback.Create;
  data := CreateSimpleData;
  try
    calc := TSkyPerfCalc.Create(callback, data, 'filename');
    results := DoRequest(calc, [1]);
    try
      CheckEquals(1, results.Count);
      CheckEqualsString('Pack1', GetResultText(results.Items[0] as TResult));
    finally
      results.Free;
    end;
  finally
    data.Free;
  end;
end;

procedure TSkyPerfCalcTests.TestEnumerateSimple2;
var
  callback: IMessageCallback;
  data: TStringList;
  calc: TSkyPerfCalc;
  results: TObjectList;
begin
  callback := TCallback.Create;
  data := CreateSimpleData;
  try
    calc := TSkyPerfCalc.Create(callback, data, 'filename');
    results := DoRequest(calc, [2]);
    try
      CheckEquals(1, results.Count);
      CheckEqualsString('2', GetResultText(results.Items[0] as TResult));
    finally
      results.Free;
    end;
  finally
    data.Free;
  end;
end;

function TSkyPerfCalcTests.CreateDataWithLimited: TStringList;
var
  data: TStringList;
begin
  data := TStringList.Create;
  data.Add(';comment');
  data.Add('S,1,100,0,Service1,123,SVC1');

//C,`lԍ,,(0ȂPƉs)
//C,`lԍ+bias, O, , sT[rX̑Ή`l(Ȃ0)
  data.Add('C,1,Ch1,100');
  data.Add('C,2,Ch2,200');
  data.Add('C,3,Ch3,300');
  data.Add('C,4,Ch4,400');
  data.Add('C,5,Ch5,500');

//P,pbN,,\`l/pbN,...
//P,pbN,,\pbN,*`l
  data.Add('P,FullPack,1300,1,2,3,4,5');
  data.Add('P,LimitedPack,700,*3,FullPack');

  result := data;

{
wanted: 1 2 3
: 1
 P: 600yen
  1 100
  2 200
  3 300
 limited pack: 700yen(1,2,3) -> Pi荂̂Ŕr
 full pack: 1300yen(1,2,3) -> Pi荂̂Ŕr

wanted: 1 2 3 4
: 2
 P: 1000yen
  1 100
  2 200
  3 300
  4 400
 limited pack: 800yen
  limited pack: 700yen
  1 100yen

 full pack: 1300yen(1,2,3,4) -> Pi荂̂Ŕr

wanted: 1 2 3 4 5
: 3
 P: 1500yen
  1 100
  2 200
  3 300
  4 400
  5 500
 limited pack: 1000yen
  limited pack: 700yen
  1 100yen
  2 200yen

 full pack: 1300yen(1,2,3,4,5)
}
end;

procedure TSkyPerfCalcTests.TestEnumerateLimited123;
var
  callback: IMessageCallback;
  data: TStringList;
  calc: TSkyPerfCalc;
  results: TObjectList;
begin
  callback := TCallback.Create;
  data := CreateDataWithLimited;
  try
    calc := TSkyPerfCalc.Create(callback, data, 'filename');

    results := DoRequest(calc, [1,2,3]);
    try
      CheckEquals(1, results.Count);
      CheckEqualsString('1,2,3', GetResultText(results.Items[0] as TResult));
    finally
      results.Free;
    end;

  finally
    data.Free;
  end;
end;

procedure TSkyPerfCalcTests.TestEnumerateLimited1234;
var
  callback: IMessageCallback;
  data: TStringList;
  calc: TSkyPerfCalc;
  results: TObjectList;
  iResult: TResult;
begin
  callback := TCallback.Create;
  data := CreateDataWithLimited;
  try
    calc := TSkyPerfCalc.Create(callback, data, 'filename');

    results := DoRequest(calc, [1,2,3,4]);
    try
      CheckEquals(2, results.Count);

      iResult := results.Items[0] as TResult;
      CheckEquals(800+123, iResult.TotalCharge);
      CheckEqualsString('1,LimitedPack', GetResultText(iResult) );
      iResult := results.Items[1] as TResult;
      CheckEquals(1000+123, iResult.TotalCharge);
      CheckEqualsString('1,2,3,4', GetResultText(iResult) );
    finally
      results.Free;
    end;

  finally
    data.Free;
  end;
end;

procedure TSkyPerfCalcTests.TestEnumerateLimited12345;
var
  callback: IMessageCallback;
  data: TStringList;
  calc: TSkyPerfCalc;
  results: TObjectList;
  iResult: TResult;
begin
  callback := TCallback.Create;
  data := CreateDataWithLimited;
  try
    calc := TSkyPerfCalc.Create(callback, data, 'filename');

    results := DoRequest(calc, [1,2,3,4,5]);
    try
      CheckEquals(3, results.Count);

      iResult := results.Items[0] as TResult;
      CheckEquals(1000+123, iResult.TotalCharge);
      CheckEqualsString('1,2,LimitedPack', GetResultText(iResult) );
      iResult := results.Items[1] as TResult;
      CheckEquals(1300+123, iResult.TotalCharge);
      CheckEqualsString('FullPack', GetResultText(iResult) );
      iResult := results.Items[2] as TResult;
      CheckEquals(1500+123, iResult.TotalCharge);
      CheckEqualsString('1,2,3,4,5', GetResultText(iResult) );
    finally
      results.Free;
    end;
  finally
    data.Free;
  end;
end;

function TSkyPerfCalcTests.CreateDataWithCombinationDiscount: TStringList;
var
  data: TStringList;
begin
  data := TStringList.Create;
  data.Add(';comment');
  data.Add('S,1,100,0,Service1,1,SVC1');

// gݍ킹Ή
//  ̒Ɋʂň͂Ń`l܂͗/pbNƁA
//  ̃`l܂̓pbNIĂꍇɗ҂ɒu

//C,`lԍ,,(0ȂPƉs)
//C,`lԍ+bias, O, , sT[rX̑Ή`l(Ȃ0)
  data.Add('C,1,Ch1,100');
  data.Add('C,2,Ch2,200');
  data.Add('C,3,Ch3,300(Pack1/200)');

//P,pbN,,\`l/pbN,...
//P,pbN,,\pbN,*`l
  data.Add('P,Pack1,250,1,2');
  data.Add('P,Pack2,400("Pack1"/300),2,3');
  data.Add('P,Pack3,490,1,2,3');
  data.Add('P,Pack4,100,Pack1,*1');

  result := data;

{
wanted: 1 2 3
: 7
-'3,Pack1': 450yen
  Pack1 250
  Ch3 300ł͂Ȃ200
-'Pack3': 490yen
  Pack3 490
-'1,Pack2': 500yen
  Pack2 400
  Ch1 100
-'Pack1,Pack2': 550yen
  Pack1 250
  Pack2 400ł͂Ȃ300
-'1,2,3': 600yen
  Ch1 100
  Ch2 200
  Ch3 300
-'1,3,Pack4': 400yen
  Ch1 100
  Ch3 300->200
  Pack4 100 (Ch2)
-'Pack2,Pack4': 400yen
  Pack2 400->300 (Ch2,Ch3)
  Pack4 100      (Ch1)
}
end;

procedure TSkyPerfCalcTests.TestEnumerateCombinationDiscount;
var
  callback: IMessageCallback;
  data: TStringList;
  calc: TSkyPerfCalc;
  results: TObjectList;
  r: TResult;
  rt: string;
  expectedcount: integer;
begin
  callback := TCallback.Create;
  data := CreateDataWithCombinationDiscount;
  try
    calc := TSkyPerfCalc.Create(callback, data, 'filename');

    results := DoRequest(calc, [1,2,3]);
    try
      //CheckEquals(5, results.Count, 'results.Count');
      expectedcount := 0;

      rt := '1,2,3';
      Check(FindResultText(r, results, rt), rt);
      CheckEquals(600+1, r.TotalCharge, rt);
      Inc(expectedcount);

      rt := '1,Pack2';
      Check(FindResultText(r, results, rt), rt);
      CheckEquals(500+1, r.TotalCharge, rt);
      Inc(expectedcount);

      rt := '3,Pack1';
      Check(FindResultText(r, results, rt), rt);
      CheckEquals(450+1, r.TotalCharge, rt);
      Inc(expectedcount);

      rt := 'Pack3';
      Check(FindResultText(r, results, rt), rt);
      CheckEquals(490+1, r.TotalCharge, rt);
      Inc(expectedcount);

      rt := '1,3,Pack4';
      Check(FindResultText(r, results, rt), rt);
      CheckEquals(400+1, r.TotalCharge, rt);
      Inc(expectedcount);

      rt := 'Pack2,Pack4';
      Check(FindResultText(r, results, rt), rt);
      CheckEquals(400+1, r.TotalCharge, rt);
      Inc(expectedcount);

      rt := 'Pack1,Pack2';
      Check(FindResultText(r, results, rt), rt);
      CheckEquals(550+1, r.TotalCharge, rt);
      Inc(expectedcount);

      if results.Count > expectedcount then
      begin
        CheckEqualsString('', GetResultText(results[expectedcount] as TResult));
      end;
      CheckEquals(expectedcount, results.Count, 'results.Count');
    finally
      results.Free;
    end;
  finally
    data.Free;
  end;
end;

procedure TSkyPerfCalcTests.TestLoad;
var
  callback: IMessageCallback;
  data: TStringList;
  calc: TSkyPerfCalc;
  service: TService;
  channel: TChannel;
  pack: TPack;
const
  FirstComment = '; first comment';
begin
  callback := TCallback.Create;

  data := TStringList.Create;
  data.Add(FirstComment);

//S,`lID͈(ŏ,ő),`l\oCAX,T[rX, {,ȗ
  data.Add('S,1,100,0,Service1,123,SVC1');
  data.Add('S,101,200,100,Service2,456,SVC2');

//C,`lԍ,,(0ȂPƉs)
//C,`lԍ+bias, O, , sT[rX̑Ή`l(Ȃ0)
  data.Add('C,1,Ch1,0'); // Ch1(SVC1)͒PƉs
  data.Add('C,2,Ch2,100'); // Ch2(SVC1)100~
  data.Add('C,102,Ch2,105,2'); // Ch2(SVC2)105~, Ή`l2

//P,pbN,,\`l/pbN,...
//P,pbN,,\pbN,*`l
  data.Add('P,Pack1,200,1,2');
  data.Add('P,Pack2,150,*1,Pack1');

  try
    calc := TSkyPerfCalc.Create(callback, data, 'filename');
    try
      CheckEqualsString(FirstComment, calc.DataVersionComment);
      CheckEquals(2, calc.ServiceCount);

      service := calc.Services[0];
      CheckEqualsString('Service1', service.Name);
      CheckEqualsString('SVC1', service.ShortName);
      CheckEquals(123, service.Charge);
      CheckEquals(2, service.ChannelCount);
       channel := service.Channels[0];
       CheckEquals(1, channel.Num);
       CheckEqualsString('Ch1', channel.Name);
       CheckEquals(0, channel.Charge);
       channel := service.Channels[1];
       CheckEquals(2, channel.Num);
       CheckEqualsString('Ch2', channel.Name);
       CheckEquals(100, channel.Charge);

       channel := calc.FindChannel(1);
       CheckEquals(1, channel.Num);
       CheckEqualsString('Ch1', channel.Name);
       channel := calc.FindChannelByName('Ch1');
       CheckEquals(1, channel.Num);
       CheckEqualsString('Ch1', channel.Name);
       channel := calc.FindChannel(2);
       CheckEquals(2, channel.Num);
       CheckEqualsString('Ch2', channel.Name);
       channel := calc.FindChannelByName('Ch2');
       CheckEquals(2, channel.Num);
       CheckEqualsString('Ch2', channel.Name);

      service := calc.Services[1];
      CheckEqualsString('Service2', service.Name);
      CheckEqualsString('SVC2', service.ShortName);
      CheckEquals(456, service.Charge);
      CheckEquals(1, service.ChannelCount);
       channel := service.Channels[0];
       CheckEquals(102, channel.Num);
       CheckEqualsString('Ch2', channel.Name);
       CheckEquals(105, channel.Charge);
       CheckEquals(2, channel.link_ch);

      CheckEquals(2, calc.PackCount);
      pack := calc.Packs[0];
       CheckEqualsString('Pack1', pack.Name);
       CheckEquals(200, pack.Charge);
       CheckEquals(2, pack.ChannelCount);
       CheckTrue( pack.Contains(1) );
       CheckTrue( pack.Contains(2) );
       CheckFalse( pack.Contains(102) );
       CheckFalse(pack.Limited);
      pack := calc.Packs[1];
       CheckEqualsString('Pack2', pack.Name);
       CheckEquals(150, pack.Charge);
       CheckEquals(2, pack.ChannelCount);
       CheckTrue(pack.Limited);
       CheckEquals(1, pack.Limit);
       CheckTrue( pack.Contains(1) );
       CheckTrue( pack.Contains(2) );
    finally
      calc.Free;
    end;
  finally
    data.Free;
  end;
end;

{ TCallback }

procedure TCallback.AddLine(s: string);
begin
  FBuffer := FBuffer + s + #13#10;
end;

procedure TCallback.Clear;
begin
  FBuffer := '';
end;

constructor TCallback.Create;
begin

end;

function TCallback.Get: string;
begin
  result := FBuffer;
end;

{ TChargeFieldTests }

procedure TChargeFieldTests.TestCreate;
var
  a: TChargeField;
begin
  a := TChargeField.Create('200');
  try
    CheckEquals(200,a.Charge);
    CheckEqualsString('',a.DiscountWithName);
    CheckEquals(200,a.DiscountCharge);
  finally
    a.free;
  end;

  a := TChargeField.Create('100(hoge/10)');
  try
    CheckEquals(100,a.Charge);
    CheckEqualsString('hoge',a.DiscountWithName);
    CheckEquals(10,a.DiscountCharge);
  finally
    a.free;
  end;

  a := TChargeField.Create('100("hoge/hoge"/10)');
  try
    CheckEquals(100,a.Charge);
    CheckEqualsString('hoge/hoge',a.DiscountWithName);
    CheckEquals(10,a.DiscountCharge);
  finally
    a.free;
  end;
end;

{ TIntSetTests }

procedure TIntSetTests.SetUp;
begin
  inherited;
  FObject := TIntSet.Create;
end;

procedure TIntSetTests.TearDown;
begin
  inherited;
  FObject.Free;
end;

procedure TIntSetTests.TestAdd_Count;
begin
  CheckEquals(0, FObject.Count);
  FObject.Add(10);
  CheckEquals(1, FObject.Count);
  FObject.Add(30);
  CheckEquals(2, FObject.Count);
  FObject.Add(20);
  CheckEquals(3, FObject.Count);

  // łɂl𑫂ĂȂ
  FObject.Add(20);
  FObject.Add(10);
  FObject.Add(30);
  CheckEquals(3, FObject.Count);

  // ŏlAől̊O̒l𑫂Ăok
  FObject.Add(5);
  CheckEquals(4, FObject.Count);
  FObject.Add(40);
  CheckEquals(5, FObject.Count);
end;

procedure TIntSetTests.TestAddSet_RemoveSet;
var
  obj2: TIntSet;
begin
  FObject.Add(10);
  FObject.Add(20);
  FObject.Add(30);
  CheckEquals(3, FObject.Count);

  obj2 := TIntSet.Create;
  try
    obj2.Add(30);
    obj2.Add(40);
    obj2.Add(50);
    CheckEquals(3, obj2.Count);

    FObject.AddSet(obj2);
    CheckEquals(5, FObject.Count);

    FObject.RemoveSet(obj2);
    CheckEquals(2, FObject.Count);
  finally
    obj2.Free;
  end;
end;

procedure TIntSetTests.TestAssign_Next;
var
  obj2: TIntSet;
  val: integer;
  val2: integer;
begin
  FObject.Add(10);
  FObject.Add(20);
  FObject.Add(30);
  CheckEquals(3, FObject.Count);

  // normal Next
  val := 5;
  CheckTrue(FObject.Next(val));
  CheckEquals(10, val);
  CheckTrue(FObject.Next(val));
  CheckEquals(20, val);
  CheckTrue(FObject.Next(val));
  CheckEquals(30, val);
  CheckFalse(FObject.Next(val));

  // rő݂Ȃ̂Ă̎
  val := 25;
  CheckTrue(FObject.Next(val));
  CheckEquals(30, val);

  // ől傫false
  val := 40;
  CheckFalse(FObject.Next(val));

  // Assign
  obj2 := TIntSet.Create;
  try
    obj2.Assign(FObject);
    CheckEquals(3, obj2.Count);
    val := -1;
    val2 := -1;
    while FObject.Next(val) do
    begin
      CheckTrue(obj2.Next(val2));
      CheckEquals(val, val2);
    end;
    CheckFalse(obj2.Next(val2));
  finally
    obj2.Free;
  end;
end;

procedure TIntSetTests.TestClear;
begin
  FObject.Add(10);
  FObject.Add(20);
  FObject.Add(30);
  CheckEquals(3, FObject.Count);
  CheckFalse(FObject.IsEmpty);

  FObject.Clear;
  CheckEquals(0, FObject.Count);
  CheckTrue(FObject.IsEmpty);
end;

procedure TIntSetTests.TestCreate;
var
  val: integer;
begin
  CheckEquals(0, FObject.Count);
  CheckTrue(FObject.IsEmpty);
  val := -1;
  CheckFalse(FObject.Next(val));
end;

procedure TIntSetTests.TestFirst_Last;
var
  val: integer;
begin
  CheckTrue(FObject.IsEmpty);
  CheckFalse(FObject.First(val));
  CheckFalse(FObject.Last(val));

  FObject.Add(10);
  CheckFalse(FObject.IsEmpty);
  val := -1;
  CheckTrue(FObject.First(val));
  CheckEquals(10, val);
  val := -1;
  CheckTrue(FObject.Last(val));
  CheckEquals(10, val);
end;

procedure TIntSetTests.TestHas;
begin
  FObject.Add(10);
  FObject.Add(20);
  FObject.Add(30);
  CheckEquals(3, FObject.Count);

  CheckFalse( FObject.Has(5) );
  CheckTrue( FObject.Has(10) );
  CheckTrue( FObject.Has(20) );
  CheckFalse( FObject.Has(25) );
  CheckTrue( FObject.Has(30) );
  CheckFalse( FObject.Has(40) );
end;

procedure TIntSetTests.TestRemove;
begin
  FObject.Add(10);
  FObject.Add(20);
  FObject.Add(30);
  CheckEquals(3, FObject.Count);
  CheckFalse(FObject.IsEmpty);

  // 
  FObject.Remove(25);
  CheckEquals(3, FObject.Count);

  // ͈͊O
  FObject.Remove(5);
  CheckEquals(3, FObject.Count);
  FObject.Remove(40);
  CheckEquals(3, FObject.Count);

  FObject.Remove(20);
  CheckEquals(2, FObject.Count);
  CheckFalse(FObject.IsEmpty);

  // x폜ĂωȂ
  FObject.Remove(20);
  CheckEquals(2, FObject.Count);
  CheckFalse(FObject.IsEmpty);

  // S폜
  FObject.Remove(10);
  CheckEquals(1, FObject.Count);
  CheckFalse(FObject.IsEmpty);
  FObject.Remove(30);
  CheckEquals(0, FObject.Count);
  CheckTrue(FObject.IsEmpty);
end;

initialization
{
  TestFramework.RegisterTest('skyperfdataTests Suite',
    TPackTests.Suite);
  TestFramework.RegisterTest('skyperfdataTests Suite',
    TRequestTests.Suite);
  TestFramework.RegisterTest('skyperfdataTests Suite',
    TResultTests.Suite);
  TestFramework.RegisterTest('skyperfdataTests Suite',
    TSkyPerfCalcTests.Suite);
}
  TestFramework.RegisterTest('TChargeFieldTests Suite',
    TChargeFieldTests.Suite);

  TestFramework.RegisterTest('TSkyPerfCalcTests Suite',
    TSkyPerfCalcTests.Suite);

  TestFramework.RegisterTest('TIntSetTests Suite',
    TIntSetTests.Suite);
end.
